@@ -18,9 +18,10 @@ def generate_initiation_interface():
     StudID = widgets.IntText( description='Student ID:', value=0 )
     button_StudID   = widgets.Button( description='Generate Parameters' )
     output = widgets.Output()
-
+    
+    global vb
     vb=widgets.VBox([StudID,button_StudID, output])
-    return vb
+    return
 
 ##########
 ##########
@@ -188,9 +189,9 @@ class homework_three():
 
         #output formula:
         # X-YZ
-        # X: was the help used? yes or no
+        # X: was the help used? [yes] (randrange(5,9) or [no] randrange(0,4)
         #    -> randomint
-        # YZ: Summ(studentID[0]+studentID[1]) <10 OR >= 10:
+        # YZ: Summ(studentID[0]+studentID[1]) [<10] randrange(0,4)) OR [>= 10] randrange(5,9)):
         #    -> 2x randomint
         if self.stdID == "testing":
             with self.question_one_display.children[0]:
@@ -295,8 +296,8 @@ class homework_three():
 
                 #3. Calculating Yields, YXO, YXC
                 mu = 0.35 #Growthrate = 0.35 1/h
-                Yxo = rO2/mu
-                Yxc = rCO2/mu
+                Yxo = round(rO2/mu,2)
+                Yxc = round(rCO2/mu,2)
                 Yxn = self.newN
                 print("\nYxo:",Yxo)
                 print("\nYxc:",Yxc)
@@ -325,8 +326,8 @@ class homework_three():
 
                 #3. Calculating Yields, YXO, YXC
                 mu = 0.35 #Growthrate = 0.35 1/h
-                Yxo = rO2/mu
-                Yxc = rCO2/mu
+                Yxo = round(rO2/mu,2)
+                Yxc = round(rCO2/mu,2)
                 Yxn = self.newN
 
                 Ymatrixvar = np.array([[2,1],[1,-0.5]])
@@ -539,7 +540,7 @@ class homework_three():
             "* First, convert the biomass concentration from g/L to c-mol\n" \
             "  and normalize the gas exchange rates\n\n" \
             "* You can use the growth rates and exchange rates to determine\n the yields of O_2 and CO_2\n\n" \
-            "* How to solve the matrix can be seen in 'Example 1' (p. 19/20 of Lecture 4)\n"
+            "* How to solve the matrix can be seen in 'Example 1' (p. 19/20 of Lecture QM3 ‘Quantitative Microbiologie week 4 – Folien’)\n"
             "##########\n"
             print(txt)
     def on_question_two_answer_button_clicked(self, click):
@@ -681,7 +682,7 @@ class homework_four():
                 
                 print(self.newCoP * mp)
                 print(self.newCoC * mc)
-                print(self.newCof * mf)
+                print(self.newCoF * mf)
         
         
         if type(self.stdID) == int:
@@ -760,9 +761,9 @@ class homework_four():
         #format: 
         #Y_cs = {A} + {B}*Y_co + {C}*Y_cn"
         #sol_Ycs= [A, B, C]
-        sol_Ycs = [3.4,-2.4,-3.5]
-        sol_Ycf = [-2.4,-2.4,0.3]
-        sol_Ycp = [0.0,0.0,3.8]
+        sol_Ycs = [-2.38,3.38,-3.58]
+        sol_Ycf = [2.38,-2.38,-0.27]
+        sol_Ycp = [0.0,0.0,3.85]
         
         
         if self.stdID == "testing":
@@ -832,6 +833,10 @@ class homework_four():
         #newOCR, newCPR, newNSR: changed parameters based on the seed.
             #are needed to calculate the opimal solution
         #hq3tgl: boolean. Decides if the student used the help-option or not
+        
+        #ALSO:
+        # Instead of a literature solution matrix, the student solution of Question 2 is to be used as a basis
+        # So that they can get points for Question 3, even if they got Question 2 wrong
 
         #output formula:
         # X-YZ
@@ -847,9 +852,9 @@ class homework_four():
         #format: 
         #Y_cs = {A} + {B}*Y_co + {C}*Y_cn"
         #sol_Ycs= [A, B, C]
-        sol_Ycs = [3.4,-2.4,-3.5]
-        sol_Ycf = [-2.4,-2.4,0.3]
-        sol_Ycp = [0.0,0.0,3.8]
+        #sol_Ycs = [-2.38,3.38,-3.58]
+        #sol_Ycf = [2.38,-2.38,-0.27]
+        #sol_Ycp = [0.0,0.0,3.85]     
         
         mp = 12+1.57+16*0.32+14*0.26 #CH_1.57O_0.32N_0.26
         mc = 12+2+16 #CH_2O
@@ -872,7 +877,14 @@ class homework_four():
                 Ycf = round(sol_Ycf[0]*Ycc+sol_Ycf[1]*Yco+sol_Ycf[2]*Ycn,2) #(mol/cmol)
                 Ycp = round(sol_Ycp[0]*Ycc+sol_Ycp[1]*Yco+sol_Ycp[2]*Ycn,2) #(mol/cmol)
                 
-                Yxq = Ycs*round(mp*self.newCoP,2)+Ycf*round(mf*self.newCoF,2)+Ycp*round(mp*self.newCoP,2) #(mol/cmol * kcal/cmol) = (kcal/cmol) || units not clear, might need second look
+                print('Ycs: \n',Ycs)
+                print('Ycf: \n',Ycf)
+                print('Ycp: \n',Ycp)
+                print('mc: \n', mc)
+                print('mf: \n', mf)
+                print('mp: \n', mp)
+                
+                Yxq = Ycs*round(mc*self.newCoC,2)+Ycf*round(mf*self.newCoF,2)+Ycp*round(mp*self.newCoP,2) #(mol/cmol * kcal/cmol) = (kcal/cmol) || units not clear, might need second look
                 Rxq = round(Yxq * self.newCPR,2) # kcal/cmol * mol/h = kcal/h || units not clear, might need second look
                 
                 print('Yxq: \n',Yxq)
@@ -885,6 +897,32 @@ class homework_four():
             #Calculation of the solution:
             with self.question_three_display.children[-1]:
                 
+                # Instead of a literature solution, the student solution is to be used as a basis
+                # So that they can get points for Question 3, even if they got Question 2 wrong
+                sol_Ycs = []
+                sol_Ycf = []
+                sol_Ycp = []
+                for i in self.question_two_display.children[2].children:
+                    sol_Ycs.append(i.value)
+                for i in self.question_two_display.children[4].children:
+                    sol_Ycf.append(i.value)
+                for i in self.question_two_display.children[6].children:
+                    sol_Ycp.append(i.value)
+                #RUDIMENTARY check if there are problems with the student solution of Question 2
+                checkcounter = 0
+                if sum(sol_Ycs) == 0:
+                    checkcounter += 1
+                if sum(sol_Ycf) == 0:
+                    checkcounter += 1
+                if sum(sol_Ycp) == 0:
+                    checkcounter += 1
+                    
+                if checkcounter > 0:
+                    print("\n##########\nNo Solution of Question 2 detected!\nPlease try to solve Question 2 before answering Question 3!\n\n")
+                    self.question_three_display.children[-1].clear_output(wait=True)
+                    return
+                    
+                #if Question 2 answers are accepted, continue here    
                 Ycc = round(self.newCPR/self.newCPR,2) #(mol/h / mol/h)
                 Yco = round(self.newOCR/self.newCPR,2) #(mol/h / mol/h)
                 rn = self.newNSR/14 #(Secretion rate NH3 in gN/h / 14 g/mol)
@@ -905,7 +943,7 @@ class homework_four():
                 Ycp = round(sol_Ycp[0]*Ycc+sol_Ycp[1]*Yco+sol_Ycp[2]*Ycn,2) #(mol/cmol)
                 
 
-                Yxq = Ycs*round(mp*self.newCoP,2)+Ycf*round(mf*self.newCoF,2)+Ycp*round(mp*self.newCoP,2) #(mol/cmol * kcal/cmol) = (kcal/cmol) || units not clear, might need second look
+                Yxq = Ycs*round(mc*self.newCoC,2)+Ycf*round(mf*self.newCoF,2)+Ycp*round(mp*self.newCoP,2) #(mol/cmol * kcal/cmol) = (kcal/cmol) || units not clear, might need second look
                 Rxq = round(Yxq * self.newCPR,2) # kcal/cmol * mol/h = kcal/h || units not clear, might need second look
                 
                 #Check second Checkpoint
@@ -1060,7 +1098,7 @@ class homework_four():
             # Widgtes responsible for student input of Y_cs
             ###
             listofwidgets.append(widgets.Label(
-                value="Yield Carbohydrates / Y_cs [mol/cmol]= {A} + {B}*Y_co + {C}*Y_cn",
+                value="Yield: Y_cs [mol/cmol]= {A} + {B}*Y_co + {C}*Y_cn",
                 display='flex',
                 flex_flow='column',
                 align_items='stretch',
@@ -1104,7 +1142,7 @@ class homework_four():
             # Widgtes responsible for student input of Y_cf
             ###
             listofwidgets.append(widgets.Label(
-                value="Yield Fats / Y_cf [mol/cmol]= {A} + {B}*Y_co + {C}*Y_cn",
+                value="Yield: Y_cf [mol/cmol]= {A} + {B}*Y_co + {C}*Y_cn",
                 display='flex',
                 flex_flow='column',
                 align_items='stretch',
@@ -1148,7 +1186,7 @@ class homework_four():
             # Widgtes responsible for student input of Y_cp
             ###
             listofwidgets.append(widgets.Label(
-                value="Yield Protein / Y_cp [mol/cmol]= {A} + {B}*Y_co + {C}*Y_cn",
+                value="Yield: Y_cp [mol/cmol]= {A} + {B}*Y_co + {C}*Y_cn",
                 display='flex',
                 flex_flow='column',
                 align_items='stretch',
@@ -1237,9 +1275,6 @@ class homework_four():
         listofwidgets.append(widgets.Output(
         ))
         with listofwidgets[0]:
-            #print("##########\n\nParameters for Question 1:\n\n"\
-            #"Your glucose yield Y_(XS) is {} C-mol glucose/C-mol biomass\n\n"\
-            #"Your biomass composition is CH_({})O_({})N_({})\n\n##########".format(self.newYxs,self.newH,self.newO,self.newN))
             print("")
         
         listofwidgets.append(widgets.FloatText(
@@ -1328,9 +1363,33 @@ class homework_four():
         self.hq2tgl = 1
         self.question_two_display.children[-3].children[1].disabled = True
         with self.question_two_display.children[-2]:
-            txt = "##########\n" \
-            "PLACEHOLDER!\n" \
-            "##########\n"
+            txt ="""
+##########\n
+Given is the stoichiometry of the combustion of Carbohydrates (Ycs), Fats (Ycf), and Protein (Ycp)\ntogether with Oxygen (Yco) to form CO_2 (Ycc), ammonia (Ycn) and water (Ycw).\n
+The equation is normalized on CO_2, so the yield of CO_2 (Ycc) = 1.\n
+The task is to solve Ycs, Ycf and Ycp as functions of Yco and Ycn.\n
+One way to do this, is to calculate the degree of reduction (DoR) as can be seen on p. 23-25 in the\nQM3-script ‘Quantitative Microbiologie week 4 – Folien’.\n
+Setting up a stoichiometric matrix should look like this:\n\n
+\t\t\t\t\t\t\t\tYcs\n
+\t\t\t\t\t\t\t\tYcf\n
+C  :\t1\t1\t1\t0\t0\t1\t\tYcp\n
+N  :\t0\t0\tx\t0\t1\t0\t*\tYco\t=\t0\n
+DoR:\tx\tx\tx\t4\t0\t0\t\tYcn\n
+\t\t\t\t\t\t\t\tYcc\n\n
+
+And rearranging the matrix using Gaussian elimination to get functions of Yco and Ycn should end up looking like this:\n\n
+\t\t\t\t\t\t\t\tYcs\n
+\t\t\t\t\t\t\t\tYcf\n
+C  :\t1\t0\t0 \tB_cs\tC_cs\tA_cs\t\tYcp\n
+N  :\t0\t1\t0\tB_cf\tC_cf\tA_cf\t*\tYco\t=\t0\n
+DoR:\t0\t0\t1\tB_cp\tC_cp\tA_cp\t\tYcn\n
+\t\t\t\t\t\t\t\tYcc\n\n
+Meaning that the example-equation of Ycs looks like this: \n
+Ycs + B_cs*Yco + C_cs*Ycn + A_cs*Ycc(with Ycc=1) = 0\n
+Rearrange it for Ycs and repeat the procedure for Ycf and Ycn.\n
+##########\n
+            """
+
             print(txt)
             
     def on_question_two_answer_button_clicked(self, click):
@@ -1364,7 +1423,16 @@ class homework_four():
         self.hq3tgl = 1
         self.question_three_display.children[-3].children[1].disabled = True
         with self.question_three_display.children[-2]:
-            txt = "##########\n" \
-            "PLACEHOLDER!\n" \
-            "##########\n"
+            txt ="""
+##########\n
+Task is determine the metabolic rate Rxq [kcal/h]\n
+ * First step is to calculate the Yields of O_2 (Yco) and NH_4 (Ycn)relative to CO_2\n
+(* The Yied of CO_2 (Y_cc) = 1 [mol/cmol])\n
+(* Don’t forget to calculate Ycn out of the nitrogen secretion rate!)\n
+ * Next, enter the yields into the three equations from [Question 2].\n
+ * Calculate Yxq [kJ/cmol] (p. 36-38  in the QM3-script ‘Quantitative Microbiologie week 4 – Folien’\n
+ * The formula for Rxq is Yxq [kcal/cmol] * r_co2 [cmol/h]\n
+##########\n
+            """
+
             print(txt)
\ No newline at end of file
